from PySZG import *
import time
from math import *
import random
import pdb

class navigator:
	def __init__(self):
		self.node = None
		self.lastTime = -1
		self.speed = 3.0
		self.threshold = 0.4

	def update(self, framework, time):
		if self.lastTime < 0:
			self.lastTime = time
			return
		wandDir = ar_ERM(framework.getMatrix(1))*arVector3(0,0,-1)
		wandTransDir = ar_ERM(framework.getMatrix(1))*arVector3(1,0,0)
		deltaX = framework.getAxis(0)
		deltaY = framework.getAxis(1)
		if abs(deltaX) < self.threshold:
			deltaX = 0
		else:
			deltaX = (deltaX - self.threshold) / (1.0 - self.threshold)
		if abs(deltaY) < self.threshold:
			deltaY = 0
		else:
			deltaY = (deltaY - self.threshold) / (1.0 - self.threshold)
		t = (wandDir * deltaY + wandTransDir * deltaX) * (time - self.lastTime) * self.speed 
		self.node.set(ar_TM(-t)*self.node.get())
		self.lastTime = time
class simulator(arPyInputSimulator):
    def onKeyboard(self, key, state):
        if key == 'ESCKEY' and state == 1:
            return

#This is the meathod to draw the quasicrystals (who would have thought?)
def drawQuasiCrystal(U, V, W, x, y, z, m):
    #Here we define a set op points. These points are the vertices of the quasi-crystal.
    p = m.new("points")
    p.set([arVector3(x,y,z),                                                    #0          0
           arVector3(x+U[0], y+U[1], z+U[2]),                                   #U          1
           arVector3(x+V[0], y+V[1], z+V[2]),                                   #V          2
           arVector3(x+W[0], y+W[1], z+W[2]),                                   #W          3
           arVector3(x+U[0]+V[0], y+U[1]+V[1], z+U[2]+V[2]),                    #UV         4
           arVector3(x+U[0]+W[0], y+U[1]+W[1], z+U[2]+W[2]),                    #UW         5
           arVector3(x+W[0]+V[0], y+W[1]+V[1], z+W[2]+V[2]),                    #VW         6
           arVector3(x+U[0]+V[0]+W[0], y+U[1]+V[1]+W[1], z+U[2]+V[2]+W[2])])    #UVW        7
    
    #Here we define the indices for drawing the quasicrystal.
    i1 = p.new("index")
    i1.set([0,1,4,7])
    i2 = p.new("index")
    i2.set([0,2,6,7])
    i3 = p.new("index")
    i3.set([0,3,5,7])
    i4 = p.new("index")
    i4.set([1,5])
    i5 = p.new("index")
    i5.set([2,4])
    i6 = p.new("index")
    i6.set([3,6])
    
    #Here we draw the quasicrystal using a series of line strips.
    d1 = i1.new("drawable")
    d1.setDrawable(DG_LINE_STRIP, 3)
    d2 = i2.new("drawable")
    d2.setDrawable(DG_LINE_STRIP, 3)
    d3 = i3.new("drawable")
    d3.setDrawable(DG_LINE_STRIP, 3)
    d4 = i4.new("drawable")
    d4.setDrawable(DG_LINE_STRIP, 1)
    d5 = i5.new("drawable")
    d5.setDrawable(DG_LINE_STRIP, 1)
    d6 = i6.new("drawable")
    d6.setDrawable(DG_LINE_STRIP, 1)

def calculatePosition(U, V, W, m):
    #This is our method to calculate the positions of the quasicrystals in
    #the lattice.Then it calls the method to draw the crystals. In this instance
    #we are using 5 planes on either side of the origin. More could be used, but
    #the program is slow enough as it is.
    i = -1                  
    while(i<2):            ##
        j = -1              #
        while(j<2):         #Make-shift nested for loops.
            k = -1          #
            while(k<2):    ##

                # we must ignore places where the indices are equal to zero.
                if i != 0 and j != 0 and k != 0:
                    #Here we find one point on each of the planes
                    xU = i*U[0]
                    yU = i*U[1]
                    zU = i*U[2]
                    cU = U[0]*xU+U[1]*yU+U[2]*zU

                    xV = j*V[0]
                    yV = j*V[1]
                    zV = j*V[2]
                    cV = V[0]*xV+V[1]*yV+V[2]*zV

                    xW = k*W[0]
                    yW = k*W[1]
                    zW = k*W[2]
                    cW = W[0]*xW+W[1]*yW+W[2]*zW
                    
                    #Now we use Cramer's Rule to calculate the positions of the qausicrystals.
                    #D is the determinant of the coefficients.
                    D = U[0]*(V[1]*W[2] - V[2]*W[1]) + U[1]*(V[0]*W[2] - V[2]*W[0]) + U[2]*(V[0]*W[1] - V[1]*W[0])

                    #If D is equal to zero, then the planes do not intersect at a single point so we ignore them.
                    if(D != 0):
                        Dx = U[1]*(V[2]*cW - cV*W[2]) + U[2]*(V[1]*cW - cV*W[1]) + cU*(V[1]*W[2] - V[2]*W[1])
                        Dy = U[0]*(V[2]*cW - cV*W[2]) + U[2]*(V[0]*cW - cV*W[0]) + cU*(V[0]*W[2] - V[2]*W[0])
                        Dz = U[0]*(V[1]*cW - cV*W[1]) + U[1]*(V[0]*cW - cV*W[0]) + cU*(V[0]*W[1] - V[1]*W[0])

                        x = Dx/D
                        y = Dy/D
                        z = Dz/D

                        #Now that we know the intersection point, we invoke the drawQuasiCrystal method
                        if i < 0:
                            if j < 0:
                                if k < 0:
                                    drawQuasiCrystal(-U,-V,-W,x,y,z,m)
                                else:
                                    drawQuasiCrystal(-U,-V,W,x,y,z,m)
                            else:
                                if k < 0:
                                    drawQuasiCrystal(-U,V,-W,x,y,z,m)
                                else:
                                    drawQuasiCrystal(-U,V,W,x,y,z,m)
                        else:
                            if j < 0:
                                if k < 0:
                                    drawQuasiCrystal(U,-V,-W,x,y,z,m)
                                else:
                                    drawQuasiCrystal(U,-V,W,x,y,z,m)
                            else:
                                if k < 0:
                                    drawQuasiCrystal(U,V,-W,x,y,z,m)
                                else:
                                    drawQuasiCrystal(U,V,W,x,y,z,m)

                
                k = k + 1
                #end inner while loop.
            j = j + 1
            #end middle while loop.
        i = i + 1
        #end outer while loop.


def drawOctahedron(m):
    p = m.new("points")
    n = p.new("normal3")
    i = n.new("index")
    d = i.new("drawable")
    p.set([arVector3(0,1,0),
           arVector3(1,0,0),
           arVector3(0,0,1),
           arVector3(-1,0,0),
           arVector3(0,0,-1),
           arVector3(0,-1,0)])

    n.set([arVector3(0,1,0),
           arVector3(1,0,0),
           arVector3(0,0,1),
           arVector3(0,-1,0),
           arVector3(-1,0,0),
           arVector3(0,0,-1),
           arVector3(0,1,0),
           arVector3(1,0,0),])
          

    i.set([0,1,2,5,3,4,0,1])
    d.setDrawable(DG_TRIANGLE_STRIP,6)

    p1=m.new("points")
    i1=p1.new("index")
    n1=i1.new("normal3")
    d1=n1.new("drawable")

    p2=m.new("points")
    i2=p2.new("index")
    n2=i2.new("normal3")
    d2=n2.new("drawable")

    p1.set([arVector3(0,1,0),
            arVector3(0,0,1),
            arVector3(-1,0,0)])
    i1.set([0,1,2])
    n1.set([arVector3(0,1,0),
            arVector3(0,0,1),
            arVector3(-1,0,0)])
    d1.set(("triangles",1))

    p2.set([arVector3(0,-1,0),
            arVector3(0,0,-1),
            arVector3(1,0,0)])

    i2.set([0,1,2])
    n2.set([arVector3(0,1,0),
            arVector3(0,0,1),
            arVector3(-1,0,0)])
    d2.set(("triangles",1))


def addLights(g):
	r = g.getRoot()
	l = r.new("light")
	light = arLight()
	light.lightID = 0
	light.position = arVector4(0,5,0,0)
	light.ambient = arVector3(0,0,0)
	light.diffuse = arVector3(0,1,0)
	light.specular = arVector3(0,0,0)
	l.setLight(light)
	l = r.new("light")
	light = arLight()
	light.lightID = 1
	light.position = arVector4(0,-5,0,0)
        light.ambient = arVector3(0,0,0)
        light.diffuse = arVector3(0,1,0)
        light.specular = arVector3(0,0,0)
	l.setLight(light)
	l = r.new("light")
    

f = arDistSceneGraphFramework()
f.setAutoBufferSwap(0)

if f.init(sys.argv) != 1:
	sys.exit()

g = f.getDatabase()
root = g.getRoot()

#Translate everything so it's in front of us :)
trans = root.new("transform")
trans.set(ar_TM(0,5,-3)*ar_SM(10))

# Create a navigator object and tell it the scene graph's navigation node.
nav = navigator()
nav.node = trans

sim = simulator()

addLights(g)

m = trans.new("material")
mat = arMaterial()
mat.diffuse = arVector3(1,1,1)
mat.specular = arVector3(0.2, 0.2, 0.2)
mat.emissive = arVector3(0.1,0.1,0.1)
mat.exponent = 100
m.set(mat)

qq = 1/sqrt(5)

A = arVector3(2*qq, 0, qq)
B = arVector3(2*qq*cos(2*pi/5), 2*qq*sin(2*pi/5), qq)
C = arVector3(2*qq*cos(4*pi/5), 2*qq*sin(4*pi/5), qq)
D = arVector3(2*qq*cos(6*pi/5), 2*qq*sin(6*pi/5), qq)
E = arVector3(2*qq*cos(8*pi/5), 2*qq*sin(8*pi/5), qq)
F = arVector3(0, 0, 1)


print("20")
calculatePosition(A,B,C,m)
print("19")
calculatePosition(A,B,D,m)
print("18")
calculatePosition(A,B,E,m)
print("17")
calculatePosition(A,B,F,m)
print("16")
calculatePosition(A,C,D,m)
print("15")
calculatePosition(A,C,E,m)
print("14")
calculatePosition(A,C,F,m)
print("13")
calculatePosition(A,D,E,m)
print("12")
calculatePosition(A,D,F,m)
print("11")
calculatePosition(A,E,F,m)
print("10")
calculatePosition(B,C,D,m)
print("9")
calculatePosition(B,C,E,m)
print("8")
calculatePosition(B,C,F,m)
print("7")
calculatePosition(B,D,E,m)
print("6")
calculatePosition(B,D,F,m)
print("5")
calculatePosition(B,E,F,m)
print("4")
calculatePosition(C,D,E,m)
print("3")
calculatePosition(C,D,F,m)
print("2")
calculatePosition(C,E,F,m)
print("1")
calculatePosition(D,E,F,m)
print("0")


if f.start() != 1:
    sys.exit()
while 1:
    nav.update(f, time.clock())
    
    f.setViewer()
    f.swapBuffers()
    time.sleep(.02)
